=== added file 'bin/java_uncaught_exception'
--- bin/java_uncaught_exception	1970-01-01 00:00:00 +0000
+++ bin/java_uncaught_exception	2010-03-26 17:49:17 +0000
@@ -0,0 +1,67 @@
+#!/usr/bin/python
+
+import sys
+
+#from apport import unicode_gettext as _
+
+def make_title(report):
+	lines = report['StackTrace'].split('\n')
+	message = lines[0].strip()
+	stackframe = lines[1].strip()
+	return '%s in %s' % (message, stackframe)
+
+def main():
+	from apport.packaging_impl import impl as packaging
+	if not packaging.enabled():
+		return -1
+
+	# read from the JVM process a sequence of key, value delimited by null
+	# bytes
+	items = sys.stdin.read().split('\0')
+	d = dict()
+	while items:
+		key = items.pop(0)
+		if not items: break
+		value = items.pop(0)
+		d[key] = value
+		
+	log = open('/tmp/log', 'w')
+	log.write(`d`)
+
+	# create report
+	import apport.report
+	import os
+
+	report = apport.report.Report(type='Crash')
+	# assume our parent is the JVM process
+	report.pid = os.getppid()
+
+	report.add_os_info()
+	report.add_proc_info()
+	# these aren't relevant because the crash was in bytecode
+	del report['ProcMaps']
+	del report['ProcStatus']
+	report.add_user_info()
+
+	# add in data which was fed to us from the JVM process
+	for key, value in d.items():
+		report[key] = value
+
+	if 'MainClassUrl' in report:
+		import urlparse
+		url = report['MainClassUrl']
+		scheme, netloc, path, params, query, fragment = urlparse.urlparse(url)
+
+		if scheme == 'file':
+			if '!/' in path:
+				# jar
+				report['ExecutablePath'] = path.split('!/', 1)
+			else:
+				report['ExecutablePath'] = path
+
+	report['Title'] = make_title(report)
+
+	report.write(open(apport.fileutils.make_report_path(report), 'w'))
+
+if __name__ == '__main__':
+	main()

=== added directory 'java'
=== added directory 'java/com'
=== added directory 'java/com/ubuntu'
=== added directory 'java/com/ubuntu/apport'
=== added file 'java/com/ubuntu/apport/ApportUncaughtExceptionHandler.java'
--- java/com/ubuntu/apport/ApportUncaughtExceptionHandler.java	1970-01-01 00:00:00 +0000
+++ java/com/ubuntu/apport/ApportUncaughtExceptionHandler.java	2010-03-26 17:49:17 +0000
@@ -0,0 +1,89 @@
+package com.ubuntu.apport;
+
+import java.io.*;
+import java.util.HashMap;
+
+public class ApportUncaughtExceptionHandler
+	implements java.lang.Thread.UncaughtExceptionHandler {
+
+	public void uncaughtException(Thread t, Throwable e) {
+		System.out.println("uncaughtException");
+		if (e instanceof ThreadDeath)
+			return;
+
+		HashMap problemReport = getProblemReport(t, e);
+		System.out.println("got problem report");
+
+		try {
+		 	Process p = new ProcessBuilder("/usr/share/apport/java_uncaught_exception").start();
+			System.out.println("started process");
+
+			OutputStream os = p.getOutputStream();
+			writeProblemReport(os, problemReport);
+			System.out.println("wrote problem report");
+
+			os.close();
+
+			try {
+				p.waitFor();
+			} catch (InterruptedException ignore) {
+				// ignored
+			}
+
+		} catch (java.io.IOException ioe) {
+			// ignored
+		}
+
+        System.err.print("Exception in thread \""
+                         + t.getName() + "\" ");
+        e.printStackTrace(System.err);
+	}
+
+	public HashMap getProblemReport(Thread t, Throwable e) {
+		HashMap problemReport = new HashMap();
+		
+		StringWriter sw = new StringWriter();
+		PrintWriter pw = new PrintWriter(sw);
+		e.printStackTrace(pw);
+		problemReport.put("StackTrace", sw.toString());
+
+		problemReport.put("MainClassUrl", mainClassUrl(e));
+
+		return problemReport;
+	}
+
+	public void writeProblemReport(OutputStream os, HashMap pr)
+		throws IOException {
+
+		StringWriter sw = new StringWriter();
+		for(Object o : pr.keySet()) {
+			String key = (String)o;
+			String value = (String)pr.get(o);
+			sw.write(key);
+			sw.write("\0");
+			sw.write(value);
+			sw.write("\0");
+		}
+		os.write(sw.toString().getBytes());
+	}
+
+	public static String mainClassUrl(Throwable e) {
+		StackTraceElement[] stacktrace = e.getStackTrace();
+		String className = stacktrace[stacktrace.length-1].getClassName();
+
+		if (!className.startsWith("/")) {
+			className = "/" + className;
+		}
+		className = className.replace('.', '/');
+		className = className + ".class";
+
+		java.net.URL classUrl =
+			new ApportUncaughtExceptionHandler().getClass().getResource(className);
+
+		return classUrl.toString();
+    }
+
+	public static void install() {
+		Thread.setDefaultUncaughtExceptionHandler(new ApportUncaughtExceptionHandler());
+	}
+}

=== added file 'test/crash.java'
--- test/crash.java	1970-01-01 00:00:00 +0000
+++ test/crash.java	2010-03-26 17:49:17 +0000
@@ -0,0 +1,8 @@
+import com.ubuntu.apport.*;
+
+class crash {
+    public static void main(String[] args) {
+		com.ubuntu.apport.ApportUncaughtExceptionHandler.install();
+		throw new RuntimeException("Can't catch this");
+    }
+}

